/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the examples of the QtSerialBus module.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "settingsdialog.h"
#include "writeregistermodel.h"

#include <QModbusTcpClient>
#include <QModbusRtuSerialMaster>
#include <QStandardItemModel>
#include <QStatusBar>
#include <QUrl>
#include <QDebug>

enum ModbusConnection {
    Serial,
    Tcp
};


void MainWindow::onParameterChanged (QString name, QVariant value)
{
    if (name == "connection")
    {
        if (value.toBool())
        {
            ui->connectButton->setText("Disconnect");
            mConnection=true;
        }
        else
        {
            ui->connectButton->setText("Connect");
            mConnection=false;
        }
    }
    else if (name == "reply")
    {
        ui->readValue->addItem(value.toString());
    }
}

void MainWindow::onErrorOccured (QString error_text)
{
     statusBar()->showMessage(error_text);
}

void MainWindow::onConnectButtonClicked()
{
    statusBar()->clearMessage();
    for(auto m : mMasters)
    {
        m->setSettings(ui->portEdit->text(), m_settingsDialog->settings());
        if (m->slaveId() == 0x03)
        {
            m->setRequest(QModbusDataUnit(QModbusDataUnit::HoldingRegisters, 1024, 125));
        }
        else if (m->slaveId() == 0x06)
        {
            m->setRequest(QModbusDataUnit(QModbusDataUnit::HoldingRegisters, 0, 4));
        }
        else if (m->slaveId() == 0x08)
        {
            m->setRequest(QModbusDataUnit(QModbusDataUnit::HoldingRegisters, 0, 17));
        }
        m->connectDevice();
    }
    return;
}



void MainWindow::onReadButtonClicked()
{
//   qDebug() << Q_FUNC_INFO << QThread::currentThreadId();
    if (!mConnection)
        return ;

    if (!mPolling)
    {
        for (auto m: mMasters)
             emit m->startRead();
        mPolling = true;
        ui->readButton->setText("Stop reading");
    }
    else
    {
       for (auto m: mMasters)
          emit m->stopRead();
        mPolling = false;

        ui->readButton->setText("Read");
    }

    return ;
}

void MainWindow::initActions()
{
    ui->actionConnect->setEnabled(true);
    ui->actionDisconnect->setEnabled(false);
    ui->actionExit->setEnabled(true);
    ui->actionOptions->setEnabled(true);

    connect(ui->connectButton, &QPushButton::clicked,
            this, &MainWindow::onConnectButtonClicked);
    connect(ui->actionConnect, &QAction::triggered,
            this, &MainWindow::onConnectButtonClicked);
    connect(ui->actionDisconnect, &QAction::triggered,
            this, &MainWindow::onConnectButtonClicked);
    connect(ui->readButton, &QPushButton::clicked,
            this, &MainWindow::onReadButtonClicked);

    connect(ui->actionExit, &QAction::triggered, this, &QMainWindow::close);
    connect(ui->actionOptions, &QAction::triggered, m_settingsDialog, &QDialog::show);
}

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    m_settingsDialog = new SettingsDialog(this);

    initActions();

    writeModel = new WriteRegisterModel(this);
    ui->writeTable->addItem(tr("Holding Registers"), QModbusDataUnit::HoldingRegisters);

    // lock out the serial port option
    ui->connectType->setCurrentIndex(1);

    auto model = new QStandardItemModel(10, 1, this);
    for (int i = 0; i < 10; ++i)
        model->setItem(i, new QStandardItem(QStringLiteral("%1").arg(i + 1)));

    for (int i=0; i<lSlave.size(); i++)
    {
        mMasters.append(new Master(lSlave[i]));
        connect(mMasters[i], &Master::parameterChanged, this, &MainWindow::onParameterChanged);
        connect(mMasters[i], &Master::errorOccured,     this, &MainWindow::onErrorOccured);
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}
