/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the examples of the QtSerialBus module.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "master.h"
#include "settingsdialog.h"
#include "writeregistermodel.h"

#include <QModbusTcpClient>
#include <QModbusRtuSerialMaster>
#include <QStandardItemModel>
#include <QStatusBar>
#include <QUrl>
#include <QMutexLocker>
#include <QDebug>

enum ModbusConnection {
    Serial,
    Tcp
};


void Master::setRequest  (QModbusDataUnit unit)
{
    mRequest = unit;
}

void Master::setSettings (const QString port_name, const SettingsDialog::Settings sets)
{
    QMutexLocker locker(&mMutex);
    mSerialPortName =  port_name;
    mSets = sets;
}

void Master::onConnect()
{
    if (!modbusDevice)
        return;

    if (modbusDevice->state() != QModbusDevice::ConnectedState)
    {
        if (mSerialPortName.isEmpty())  return ;

         mMutex.lock();
         /*
         modbusDevice->setConnectionParameter(QModbusDevice::SerialPortNameParameter,
                mSerialPortName);
         modbusDevice->setConnectionParameter(QModbusDevice::SerialParityParameter,
               mSets.parity);
         modbusDevice->setConnectionParameter(QModbusDevice::SerialBaudRateParameter,
                mSets.baud);
         modbusDevice->setConnectionParameter(QModbusDevice::SerialDataBitsParameter,
                mSets.dataBits);
         modbusDevice->setConnectionParameter(QModbusDevice::SerialStopBitsParameter,
                mSets.stopBits);
                */
         const QUrl url = QUrl::fromUserInput("127.0.0.1:502");
         modbusDevice->setConnectionParameter(QModbusDevice::NetworkPortParameter, url.port());
         modbusDevice->setConnectionParameter(QModbusDevice::NetworkAddressParameter, url.host());
         modbusDevice->setTimeout(mSets.responseTime);
         modbusDevice->setNumberOfRetries(mSets.numberOfRetries);
         mMutex.unlock();

         if (!modbusDevice->connectDevice())
         {
//            statusBar()->showMessage(tr("Connect failed: ") + modbusDevice->errorString(), 5000);
         }
         else
         {
//            ui->actionConnect->setEnabled(false);
//            ui->actionDisconnect->setEnabled(true);
         }
     }
     else
     {
        mTm->stop();
        modbusDevice->disconnectDevice();
     }

}

void Master::onModbusStateChanged(int state)
{
    bool connected = (state != QModbusDevice::UnconnectedState);
    qDebug() << Q_FUNC_INFO << state;

    if (state == QModbusDevice::UnconnectedState)
    {
        emit parameterChanged ("connection", connected);
    }
    else if (state == QModbusDevice::ConnectedState)
    {
        emit parameterChanged ("connection", connected);
    }
}

void Master::onSendReadRequest()
{
    if (!modbusDevice)
        return;

    if (!mRequest.isValid())
        return;

    if (mSlaveId == -1)
        return;

    if (mRequestCount != mReplyCount)
    {
        qDebug() << Q_FUNC_INFO << mSlaveId << mRequestCount << mReplyCount;
        return;
    }

    if (auto *reply = modbusDevice->sendReadRequest(mRequest, mSlaveId))
    {
        mRequestCount++;
        if (!reply->isFinished())
            connect(reply, &QModbusReply::finished, this, &Master::onReadReady);
        else
            delete reply; // broadcast replies return immediately
    }
    else
    {
        qDebug()  << Q_FUNC_INFO << tr("Read error:")  + modbusDevice->errorString();
        emit errorOccured(tr("Read error:")  + modbusDevice->errorString());
    }
}

void Master::onReadReady()
{
    auto reply = qobject_cast<QModbusReply *>(sender());
    if (!reply)
        return;

    if (reply->error() == QModbusDevice::NoError)
    {
        mReplyCount++;
//        const QModbusDataUnit unit = reply->result();
//        for (int i = 0, total = int(unit.valueCount()); i < total; ++i)
//        {
//            const QString entry = tr("Address: %1, Value: %2").arg(unit.startAddress() + i)
//                                    .arg(QString::number(unit.value(i),
//                                          unit.registerType() <= QModbusDataUnit::Coils ? 10 : 16));
//           qDebug() << Q_FUNC_INFO << mSlaveId << entry;
//           emit parameterChanged("reply", entry);
//        }
    }
    else if (reply->error() == QModbusDevice::ProtocolError)
    {
       emit errorOccured((tr("Read response error: %1 (Mobus exception: 0x%2)").
                                    arg(reply->errorString()).
                                    arg(reply->rawResult().exceptionCode(), -1, 16)));
       qDebug() << Q_FUNC_INFO << (tr("Read response error: %1 (Mobus exception: 0x%2)").
                                   arg(reply->errorString()).
                                   arg(reply->rawResult().exceptionCode(), -1, 16));
    }
    else
    {
         emit errorOccured(tr("Read response error: %1 (code: 0x%2)").
                                    arg(reply->errorString()).
                                    arg(reply->error(), -1, 16));
        qDebug() << Q_FUNC_INFO << (tr("Read response error: %1 (Mobus exception: 0x%2)").
                                    arg(reply->errorString()).
                                    arg(reply->rawResult().exceptionCode(), -1, 16));
    }

    reply->deleteLater();
}


void Master::onThreadStarted    ()
{
    qDebug() << Q_FUNC_INFO ;
    modbusDevice = new QModbusTcpClient(this);
    mTm = new QTimer();
    mTm->setInterval(250);
    connect(mTm,  &QTimer::timeout,  this, &Master::onSendReadRequest);
    connect(this, SIGNAL(startRead()), mTm, SLOT(start()));
    connect(this, SIGNAL(stopRead()), mTm, SLOT(stop()));
    connect(modbusDevice, &QModbusClient::stateChanged,
            this, &Master::onModbusStateChanged);
}


Master::Master(int slaveId)
{
    this->moveToThread(&mThread);
    mSlaveId = slaveId;
    connect(this, &Master::connectDevice, this, &Master::onConnect);
    connect(&mThread, &QThread::started, this, &Master::onThreadStarted);
    connect(&mThread, &QThread::finished, &mThread, &QObject::deleteLater);
    mThread.start();
    QThread::sleep(1);
}

Master::~Master()
{
    if (modbusDevice)
        modbusDevice->disconnectDevice();
    delete modbusDevice;


}




