/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the examples of the QtSerialBus module.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef MASTER_H
#define MASTER_H

#include "settingsdialog.h"
#include <QModbusDataUnit>
#include <QThread>
#include <QTimer>
#include <QSerialPort>
#include <QMutex>
#include <QVariant>

class QModbusClient;
class QModbusReply;

class WriteRegisterModel;

class Master : public QObject
{
    Q_OBJECT

//    struct Settings {
//        int parity = QSerialPort::EvenParity;
//        int baud = QSerialPort::Baud19200;
//        int dataBits = QSerialPort::Data8;
//        int stopBits = QSerialPort::OneStop;

//        int responseTime = 1000;
//        int numberOfRetries = 3;
//    };

signals:
    void errorOccured (const QString err_text);
    void connectDevice();
    void parameterChanged (QString param_name, QVariant param_value);
    void startRead();
    void stopRead();

public:
    explicit Master(int slaveId = -1);
    ~Master();

private:



private slots:
    void onConnect();
    void onModbusStateChanged(int state);

    void onSendReadRequest();
    void onReadReady();

    void onThreadStarted    ();

public slots:


public:
    void setRequest  (QModbusDataUnit unit);
    void setSettings (const QString port_name, const SettingsDialog::Settings sets);
    int  slaveId() {return mSlaveId;}
    bool polling() {return mTm->isActive();}

private:

    QModbusClient* modbusDevice{nullptr};
    QThread mThread;
    QTimer* mTm;
    QString mSerialPortName;
    SettingsDialog::Settings mSets;
    QMutex                   mMutex;
    QModbusDataUnit          mRequest;
    int                      mSlaveId;
    unsigned int             mRequestCount{0};
    unsigned int             mReplyCount {0};
};

#endif // MASTER_H
